% Demonstrate the path following optimization algorithm on a 3x3 example.
% The example is chosen in such a way that many (all) of the possible
% break point conditions occur
% Here we use tau=1/2 and loss is ell_1 norm.

clear all;
printoutput=true;
plotfigure=true;
filename='figure2.eps';
% start timing:
tic

% define matrix and rhs
X=[-4  3  5; -4  5  1; 4 -3  0];
y=[8; 7; -11];
numdata=size(X,1);
numvar=size(X,2);

% define groups
groups={1,[2 3]};

% set-up parameters for qrmix algorithm
stopping.lambda=0;
includetest=true;
tau=0.5;
% qrmix algorithm uses ||Ax-b||_1 instead of ||y-Xbeta||_1, so we set:
weights.left=2*(1-tau)*ones(numdata,1);
weights.right=2*tau*ones(numdata,1);
A=-X;
b=-y;

% compute minimizers:
[betalist,lambdalist,u0list,u1list,w0list,w1list,succes]=qrmix(A,b,groups,stopping,1,weights,includetest);

% compute some extra information:
numpts=size(betalist,2);% number of break points found by qrmix
maxnormlist=max(abs(betalist),[],1);%max norm of each break point
rlist=A*betalist-b*ones(1,numpts);% misfit at each break point=y-X*beta
losslist=zeros(1,numpts);
for i=1:numpts
    losslist(i)=sumrho(rlist(:,i),tau);
end

% print matrix and rhs on screen (LaTeX style):
if printoutput
    disp('X=')
    disp(array2tex(A,'frac','(',')',10^(-6)));
    disp(',\qquad y=');
    disp(array2tex(b,'frac','(',')',10^(-6)));
    disp(' ');
    disp(['Groups are $G_1=' list2tex(groups{1},'\{','\}','/') ...
        '$ and $G_2=' list2tex(groups{2},'\{','\}','/') '$']);
    disp(' ');
    disp(['$\tau=' num2str(weights.left') '~' num2str(weights.right') '$'])
    disp('  ');
end

% print table of break points and subgradients etc on screen (LaTeX style).
% In this table r=y-X*beta and w is subgradient of \sum_i\rho_\tau(r_i)
if printoutput
    openbr='(';
    closebr=')';
    type='frac';
    disp('\begin{tabular}{lccccccc}');
    disp('\# & $\hat\beta$ & $r$ & $R$ & $\lambda$ & $u$ & $w$ & Step\\ \hline')

    for i=1:numpts
        % convert numerical data to strings:
        stepnumber=num2str(i-1);
        beta=['$' list2tex((betalist(:,i)'),openbr,closebr,type) '$'];
        R=['$' list2tex(mixednorm(betalist(:,i),groups),'','',type) '$'];
        r=['$' list2tex(rlist(:,i)',openbr,closebr,type) '$'];
        lambda=list2tex(lambdalist(i),'','',type);
        
        % R-interval:
        Rstart=list2tex(mixednorm(betalist(:,i),groups),'','',type);
        if i==numpts
            Rend='--';
        else
            Rend=list2tex(mixednorm(betalist(:,i+1),groups),'','',type);
        end
        Rinterval=['$[' Rstart ',' Rend ']$'];
        
        % lambda-interval
        lambdaend=[ '[' list2tex(lambdalist(i),'','',type)];
        if i==1
            lambdastart='+\infty[';
        else
            lambdastart=[list2tex(lambdalist(i-1),'','',type) ']'];
        end
        lambdainterval=['$' lambdaend ',' lambdastart '$'];
        
        % create strings for u and w
        switch i
            case 1
                if norm(w1list(:,i))==0
                    w=['$' list2tex(w0list(:,i),openbr,closebr,type) '$'];
                else
                    w=['$' list2tex(w0list(:,i),openbr,closebr,type) ...
                        '+\lambda' list2tex(w1list(:,i),openbr,closebr,type) '$'];
                end;
                if norm(u0list(:,i))==0
                    u=['$' '\lambda^{-1}' list2tex(u1list(:,i),openbr,closebr,type) '$'];
                else
                    u=['$' list2tex(u0list(:,i),openbr,closebr,type) ...
                        '+\lambda^{-1}' list2tex(u1list(:,i),openbr,closebr,type) '$'];
                end
            case numpts
                if norm(w0list(:,i))==0
                    w=['$' '\lambda' list2tex(w1list(:,i),openbr,closebr,type) '$'];
                else
                    w=['$' list2tex(w0list(:,i),openbr,closebr,type) ...
                        '+\lambda' list2tex(w1list(:,i),openbr,closebr,type) '$'];
                end;
                if norm(u1list(:,i))==0
                    u=['$' list2tex(u0list(:,i),openbr,closebr,type)  '$'];
                else
                    u=['$' list2tex(u0list(:,i),openbr,closebr,type) ...
                        '+\lambda^{-1}' list2tex(u1list(:,i),openbr,closebr,type) '$'];
                end
            otherwise
                w='$\vdots$';
                u='$\vdots$';
        end;
        rint='$\vdots$';
        betaint='$\vdots$';
        
        wnow=['$' list2tex(w0list(:,i)+lambdalist(i)*w1list(:,i),openbr,closebr,type) '$'];
        unow=['$' list2tex(u0list(:,i)+u1list(:,i)/lambdalist(i),openbr,closebr,type) '$'];
        
        comment1='\raisebox{-3.5mm}[0pt][0pt]{$\downarrow$1a}';
        comment2='\raisebox{-3.5mm}[0pt][0pt]{$\downarrow$1a}';
        
        % add line to table (include step number)
        disp([stepnumber '& ' beta ' & ' r ' & ' R ' & ' lambdainterval  ' & '   u ' &' w  ' & 3\,' comment1 '\\']);
        % add line to table (no step number)
        if i<numpts
            disp([ '& ' betaint '& ' rint ' & ' Rinterval ' & $' lambda '$ &' unow ' & ' wnow '& 1\,' comment2 ' \\'  ]);
        end;
    end
    disp('\end{tabular}');
end;

mixednormlist=zeros(1,numpts);
for i=1:numpts
    mixednormlist(i)=mixednorm(betalist(:,i),groups);
end

if plotfigure
    fontsize=10;
    
    figure
    
    % Plot beta as a function of R
    subplot(1,2,1)
    plot(mixednormlist,betalist(1,:),'-ok',...
        mixednormlist,betalist(2,:),'--ok',...
        mixednormlist,betalist(3,:),':ok');
    axis([0 max(mixednormlist) -max(abs(betalist(:))) max(abs(betalist(:)))/2]);
    set(gca,'XTick',mixednormlist);
    for i=1:numel(mixednormlist)
        if mixednormlist(i)~=25/12
        text('Interpreter','latex','HorizontalAlignment','center',...
            'FontName','Helvetica',...
            'String',['$\mathsf{' real2tex(mixednormlist(i)) '}$'],...
            'Position',[mixednormlist(i) -1.07*max(abs(betalist(:)))],...
            'FontSize',fontsize)
        else
            text('Interpreter','latex','HorizontalAlignment','center',...
            'FontName','Helvetica',...
            'String',['$\mathsf{' real2tex(mixednormlist(i)) '}$'],...
            'Position',[mixednormlist(i)+0.2 -1.07*max(abs(betalist(:)))],...
            'FontSize',fontsize)
            
        end
    end
    set(gca,'XTickLabel','')
    set(gca,'YTick',[-2 -1, 0, 1]);
    for i=[-2 -1 0 1]
        text('Interpreter','latex','HorizontalAlignment','right',...
            'FontName','Helvetica','Position',[-0.1 i],'FontSize',fontsize,...
            'String',['$\mathsf{' real2tex(i) '}$']);
    end
    set(gca,'YTickLabel','')
    % horizontal label
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica', 'FontSize',fontsize,...
        'Position',[1.02*(mixednormlist(1)+mixednormlist(end))/2 -1.07*max(abs(betalist(:)))],...
        'String','$\mathsf{R}$');
    % vertical label
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica','Position',[-0.7 -1.4],...
        'String','\textsf{Parameters} $\mathsf{\hat\beta}$',...
        'Rotation',90, 'FontSize',fontsize)
    % line labels
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica','String','$\mathsf{\hat\beta_1}$',...
        'Position',[4 -3.3],'FontSize',fontsize)
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica','String','$\mathsf{\hat\beta_2}$',...
        'Position',[4 -1.3],'FontSize',fontsize)
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica','String','$\mathsf{\hat\beta_3}$',...
        'Position',[4 0],'FontSize',fontsize)
    pbaspect([1 1 1])
    
    
    % plot loss as a function of R
    subplot(1,2,2)
    plot(mixednormlist,losslist,'-ok');%
    set(gca,'XTick',mixednormlist);
    for i=1:numel(mixednormlist)
        if mixednormlist(i)~=25/12
        text('Interpreter','latex','HorizontalAlignment','center',...
            'FontName','Helvetica',...
            'String',['$' real2tex(mixednormlist(i)) '$'],...
            'Position',[mixednormlist(i) -1.6],...
            'FontSize',fontsize)
        else
            text('Interpreter','latex','HorizontalAlignment','center',...
            'FontName','Helvetica',...
            'String',['$\mathsf{' real2tex(mixednormlist(i)) '}$'],...
            'Position',[mixednormlist(i)+0.2 -1.6],...
            'FontSize',fontsize)
            
        end
    end
    set(gca,'XTickLabel','')
    set(gca,'YTick',losslist(end:-1:1));
    for i=numel(mixednormlist):-1:1
        if abs(losslist(i)-23/6)>0.001
        text('Interpreter','latex','HorizontalAlignment','right',...
            'FontName','Helvetica',...
            'String',['$\mathsf{' real2tex(losslist(i)) '}$'],...
            'Position',[-0.15 losslist(i)],...
            'FontSize',fontsize)
        else
            text('Interpreter','latex','HorizontalAlignment','right',...
            'FontName','Helvetica',...
            'String',['$\mathsf{' real2tex(losslist(i)) '}$'],...
            'Position',[-0.15 losslist(i)-1.5],...
            'FontSize',fontsize)
        end
    end
    set(gca,'YTickLabel','')
    
    for i=1:numel(mixednormlist)-1
        text('Interpreter','latex','HorizontalAlignment','left',...
            'FontName','Helvetica',...
            'String',['$\mathsf{-\lambda=' real2tex(-lambdalist(i)) '}$'],...
            'Position',[1.02*(mixednormlist(i)+mixednormlist(i+1))/2 1.1*(losslist(i)+losslist(i+1))/2],...
            'FontSize',fontsize)
    end
    % horizontal label
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica',...
        'String','$\mathsf{R}$',...
        'Position',[1.02*(mixednormlist(1)+mixednormlist(end))/2 -1.3],...
        'FontSize',fontsize)
    % vertical label
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica',...
        'String','$\mathsf{Loss}$',...
        'Position',[-0.5 (losslist(1)+losslist(end))/2],...
        'Rotation',90,...
        'FontSize',fontsize)
    pbaspect([1 1 1])
    saveas(gcf,filename)
end;


timespent=toc;
disp(['Time spent: ' num2str(timespent)]);