% This script reads birthweight.data and reformats the contents to Matlab
% format. Final output is a design matrix X and response variables y, which
% are saved to file (LBW.mat). This LBW.mat file is needed by the other
% low birth weight (LBW) scripts of application 1.

clear all
% the data file contaning the raw data is downloaded from:
% http://www.statlab.uni-heidelberg.de/data/linmod/birthweight.html
% the variables list is as follows:
% low: indicator of birth weight less than 2.5kg
% age: mother's age in years
% lwt: mother's weight in pounds at last menstrual period
% race: mothers race ("white", "black", "other")
% smoke: smoking status during pregnancy
% ht: history of hypertension
% ui: presence of uterine irritability
% ftv: number of physician visits during the first trimester
% ptl: number of previous premature labours
% bwt: birth weight in grams

% open the text file which contains the data in above url
fid = fopen('birthweight.data');
% Read numbers and string as string into a cell array
rawdata=textscan(fid,'%s');
fclose(fid);
rawdata=rawdata{1};
% reshape into rectangular cell array (num row/colums hardcoded here)
numrows=190;
numcols=10;
rawdata=reshape(rawdata,numcols,numrows)';

% the first row=names of the variables
% rest=data
names=rawdata(1,:);
data=rawdata(2:end,:);
num_obs=size(data,1);% number of observations, =189

% make two dummy variables out of the variable 4 (race) which
% is qualitative with 3 levels. The coding is as following:
% black = (1,0), white = (0,1), and other = (0,0).
race_column=4;
race=zeros(num_obs,2);
for i=1:num_obs
    if  strcmp(data(i,race_column), 'black')
        race(i,:)=[1 0];
    elseif strcmp(data(i,race_column), 'white')
        race(i,:)=[0 1];
    end
end

% convert strings to numbers,
% race becomes NaN
numericdata=str2double(data);

% isolate response variable (last column)
y=numericdata(:,end);

%  set up design matrix X
X=NaN(num_obs,12);
% now fill columns of X:
X(:,1)=ones(num_obs,1);% for estimating the intercept of the model
X(:,2)=numericdata(:,2);% age
X(:,3)=numericdata(:,2).^2;% age^2
X(:,4)=numericdata(:,3);% weight
X(:,5)=numericdata(:,3).^2;% weight^2
X(:,6:7)=race;% race
X(:,8)=numericdata(:,5);% smoke
X(:,9)=numericdata(:,6);% ht
X(:,10)=numericdata(:,7);% ui
X(:,11)=numericdata(:,8);% ftv
X(:,12)=numericdata(:,9);% ptl
% The first column in the original data indicates if each infant's weight
% would be regarded as low or not. Since Hosmer and Lameshow used the data
% to fit a logistic regression they have needed the binary response
% variable, but here there is no need for it.


% determine the mean of quantitative variables
display('Means of quantitative variables:')
display(['Age = ', num2str(mean(X(:,2)))]);
display(['Weight = ', num2str(mean(X(:,4)))]);
display(['Number of physician visits during the first trimester = ', num2str(mean(X(:,11)))]);
display(['Number of previous premature labours = ', num2str(mean(X(:,12)))]);



% save X and y in LBW.mat
save('LBW.mat','X','y')