% application 1: Low Birth Weight data set
% This script processes the LBW full data set for many different random
% jitter to study the effect on the solution, including different size of
% jitter

clear all
numtrials=20;
epsilonlist=[0.000001 0.00001 0.0001];% size of jitter
taulist=[0.1 0.5 0.9];
savepic=true;
% use seed to be able to duplicate this result
RandStream.setDefaultStream(RandStream('mt19937ar','seed',12345));


for ind=1:numel(epsilonlist)
    epsilon=epsilonlist(ind);
    disp(['Jitter size=' num2str(epsilon)]);
    
    % load data:
    load('LBW.mat')
    % List of the loaded variables:
    % y: response variable
    % X: 12 explanatory variables after adding columns of 1's for intercept and
    % making dummy variables out of qualitative variables with number of levels more
    % than 1
    
    % defining the variables in the model
    numvar=size(X,2);
    numdata=size(X,1);
    
    % standardize the quantitative variables
    quantitative=[2 3 4 5 11 12];
    X(:,quantitative)=zscore(X(:,quantitative));
    y=zscore(y);
    
    % define groups
    numgroups=9;
    groups={1,[2,3],[4 5],[6 7], 8, 9,10,11,12};
    GRP={'1: Intercept','2: Age','3: Weight','4: Race','5: Smoking',...
        '6: Hypertens.','7: Uterine irrit.','8: Phys. visits','9: Prem. Lab.'};
    
    % set up list of tau
    numtau=numel(taulist);
    
    beta_selected=zeros(numtrials,numtau,numvar);
    X_jitt=X;
    y_jitt=y;
    disp(['Start of ' num2str(numtrials) ' trials.']);
    jitt=0;
    datestr(now)
    while jitt<numtrials
        jitt=jitt+1;
        %disp(['Trial ' num2str(jitt) ' of ' num2str(numtrials)]);
        
        % add negligible jitter to X and y
        for m=1:numvar
            jitter=randn(size(X(:,m)))*(std(X(:,m))*epsilon);
            X_jitt(:,m)=X(:,m)+jitter;
        end
        y_jitt=y+(randn(size(y))*std(y)*epsilon);
        
        % set some input variables for the algorithm which are independent of \tau
        stopping.lambda=0;
        includetest=true;
        verbose=0;
        
        % fit structured quantile regression for different tau
        flag=false;
        for i=1:numel(taulist)
            tau=taulist(i);
            % NB: qrmix computes minimizers of ||A*x-b||_1 instead of ||y-X*beta|\_1
            % (sign change = left/right change)
            weights.left=(2*tau)*ones(numdata,1);
            weights.right=(2*(1-tau))*ones(numdata,1);
            % compute solution path
            [xlist,lambdalist,u0list,u1list,w0list,w1list,succes]=qrmix(X_jitt,y_jitt,groups,stopping,verbose,weights,includetest);
            if ~succes
                % is qrmix is not succesful, then retry (with different jitter)
                disp('Trying again...')
                jitt=jitt-1;
                break;
            else
                numpts=size(xlist,2);
                k=20;% number of interpolation points between each break point in xlist
                xlist_ext=interpolatelist(xlist,groups,k);
                [BIC tmp]=BICminimizer(xlist_ext,X_jitt,y_jitt,groups,tau,false);
                beta_selected(jitt,i,:)=tmp;
            end;
        end
    end
    
    % compute relative differences with mean
    beta_mean=mean(beta_selected);
    differences=zeros(numtrials,numtau);
    for i=1:numtrials
        for k=1:numtau
            tmp1=squeeze(beta_selected(i,k,:));
            tmp2=squeeze(beta_mean(1,k,:));
            tmp3=norm(tmp2);
            differences(i,k)=norm(tmp1-tmp2)/tmp3;
        end
    end
    
    
    meandifferences(ind,:)=mean(differences);
    stdevdifferences(ind,:)=std(differences);
    
    
end%end loop over epsilon


timespent=toc;

% plot the mean difference
fontsize=24;
figure
linespeclist={'+k','*k','xk','ok'};
hold on;
for ind=1:numel(taulist)
plot(epsilonlist,meandifferences(:,ind),linespeclist{ind});
legendlist{ind}=['\tau=' num2str(taulist(ind))];
end
hold off;
ylim([10^(-7) 10^(-2)])
xlim([10^(-7) 10^(-3)])
set(gca,'XTick',taulist)
xlabel('Size of jitter','FontSize',fontsize)
ylabel('Mean difference','FontSize',fontsize)
set(gca,'yscale','log')
set(gca,'xscale','log')
set(gca,'XTick',epsilonlist)
set(gca,'YTick',[10^(-6) 10^(-4) 10^(-2)])
set(gca,'FontSize',fontsize);
legend(legendlist,'Location','SouthEast')
pbaspect([2 1 1])

if savepic saveas(gcf,'figure_jitter1.eps');end;

datestr(now)
disp('Done');