% application 1: Low Birth Weight data set
% This script processes the LBW full data set and produces a figure with the
% solution path of a group-sparse quantile regression minimization problem.
% Three values of tau are used. BIC is used to select the model

clear all
epsilon=0.0001;% size of jitter
savepic=true;% save picture at the end
% use seed to be able to duplicate this result
RandStream.setDefaultStream(RandStream('mt19937ar','seed',12345));

%%%%%%%%%%%%%%%%
% load data:
%%%%%%%%%%%%%%%%
load('LBW.mat');%
% List of the loaded variables:
% y: response variable
% X: 12 explanatory variables including a column of 1's for intercept and
% dummy variables for qualitative variable "race" (black, white, other)

% define the variables in the model
numvar=size(X,2);
VARS={'Intercept';'Age';'Age^2';'Weight';'Weight^2';'Black';'White';...
    'Smoking';'Hypertens.';...
    'Uterine irrit.';'Phys. visits';'Prem. Lab.'};%new

% standardize the quantitative variables
quantitative=[2 3 4 5 11 12]; 
X(:,quantitative)=zscore(X(:,quantitative)); 
y=zscore(y);

% define groups
numgroups=9;
groups={1,[2,3],[4 5],[6 7], 8, 9,10,11,12};
GRP={'1: Intercept','2: Age','3: Weight','4: Race','5: Smoking',...
    '6: Hypertens.','7: Uterine irrit.','8: Phys. visits','9: Prem. Lab.'};

% set tau
taulist=[0.1 0.5 0.9];
numtau=numel(taulist);
titlelist={'\tau = 0.1','\tau = 0.5','\tau = 0.9'};

% add negligible jitter to X and y
for m=1:numvar
    jitter=randn(size(X(:,m)))*(std(X(:,m))*epsilon);
    X(:,m)=X(:,m)+jitter;
end
y=y+(randn(size(y))*std(y)*epsilon);

%Preallocate cell arrays which will restore results for different \tau's
Xlist=cell(1,numtau);
Rlist=cell(1,numtau);
infnorms=cell(1,numtau);
beta_selected=cell(1,numel(taulist));
R_selected=cell(1,numel(taulist));

% set some input variables for the algorithm which are independent of tau
stopping.lambda=0;
includetest=true;
verbose=0;

% perform structured quantile regression for different tau
for i=1:numel(taulist)
    tau=taulist(i);
    % NB: qrmix computes minimizers of ||A*x-b||_1 instead of ||y-X*beta|_1
    % (sign change = left/right change)
    weights.left=(2*tau)*ones(size(X,1),1);
    weights.right=(2*(1-tau))*ones(size(X,1),1);
    % compute solution path
    disp('Start of qrmix...');
    [xlist,lambdalist,u0list,u1list,w0list,w1list,succes]=qrmix(X,y,groups,stopping,verbose,weights,includetest);
    disp('End of qrmix');
    Xlist{i}=xlist;
    % compute number of break points returned by qrmix
    numpts=size(xlist,2);
    % calculate mixed norm R for break point in xlist
    tmp=zeros(1,numpts);
    for j=1:numpts
        tmp(j)=mixednorm(xlist(:,j),groups);
    end
    Rlist{i}=tmp;
    % Compute the  \ell_{\infty} norm of \hat{\beta}'s of each group:
    tmp=zeros(numgroups,numpts);
    for j=1:numgroups
        for l=1:numpts
            tmp(j,l)=norm(xlist(groups{j},l),inf);
        end
    end
    infnorms{i}=tmp;
    % find the best model using BIC
    k=20;% number of interpolation points between each break point in xlist
    xlist_ext=interpolatelist(xlist,groups,k);
    [BIC beta_selected{i}]=BICminimizer(xlist_ext,X,y,groups,tau,true);
    % find mixed norm R for selected beta:
    R_selected{i}=mixednorm(beta_selected{i},groups);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% make figure:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
fontsize=10;
% first row: solutions paths for different tau's
plotstyles={'k',':k','--k','-.k','g',':g','--g','-.g','b'};
mx=[5 4 4];% range of horizontal axis on first row plots
for i=1:numel(taulist)
    px=0.16+((i-1)*0.23);
    subplot('Position',[px 0.7 0.15 0.27])
    current_infnorms=infnorms{i};
    current_xlist=Xlist{i};
    for j=1:numgroups
        hold on;
        plot(Rlist{i},current_infnorms(j,:),plotstyles{j});
    end
    hold off
    axis([0 mx(i) 0 1.5])
    line([R_selected{i} R_selected{i}],[0 1.5],'LineStyle','--','Color','k');
    fontsize=10;
    text('Interpreter','latex','HorizontalAlignment','left',...
        'FontName','Helvetica','String','$R_\mathrm{BIC}$',...
        'Position',[R_selected{i}+0.01 1.45],'FontSize',fontsize);
    h= text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica',...
        'String','$\|\hat\beta_{G_k}\|_\infty\,\,  (k=1,\ldots, 9)$',...
        'Rotation',90,'Position',[-1.3 0.75],'FontSize',fontsize);
    xlabel('R')
    title(titlelist{i})
end
position=[0.83 0.72 0.101 0.25];
legend(GRP{1},GRP{2},GRP{3},GRP{4},GRP{5},GRP{6},GRP{7},GRP{8},GRP{9},'Location',position)

% second row: plot \hat{\beta} for the selected model.
for i=1:3
    px=0.15+((i-1)*0.30);
    subplot('Position',[px 0.29 0.26 0.31])
    bar(beta_selected{i},'k');
    axis([0 numvar+1 -1.2 1.5])
    set(gca,'XTick',1:numvar)
    set(gca,'XTickLabel',VARS)
    xticklabel_rotate([],90,[],'Fontsize',8)
    set(gca,'FontSize',10)
    fontsize=12;
    h= text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica','String','$\hat{\beta}$',...
        'Position',[-2.22 -0.1],'FontSize',fontsize);
    set(h,'Interpreter','latex')
    title(titlelist{i})
end
% save figure to file
if savepic
    disp('Saving figure to eps file...');
    saveas(gcf,'figure3.eps');
end;
disp('Done.');



