% This script reads 93cars.dat.txt and reformats the contents to Matlab
% format. Final output is saved to file (cars93.mat):
% Y: the matrix of 5 response variables
% XX: matrix of 14 explanatory variables
% Y_z: standardized Y
% XX_z: standardized XX
% y: re-arranged Y_z as a vector 
% X: re-arranged XX_z a block-diagonal matrix
% A picture is of the response variables
% This 93cars.mat file is needed by the other 93cars scripts of application 2.

clear all
savepic=true;
% the data file contaning the raw data is downloaded from:
% http://amstat.org/publications/jse/v1n1/datasets.lock.html
% the variables list is as follows:
% 1 Manufacturer
% 2 Model
% 3 Type Small, Sporty, Compact, Midsize, Large (as defined in the Consumer Reports article)
% 4 Minimum Price (in $1,000) - Price for basic version of this model (y1)
% 5 Midrange Price (in $1,000) - Average of Min and Max prices (y2)
% 6 Maximum Price (in $1,000) - Price for a premium version (y3)
% 7 City MPG (miles per gallon by EPA rating) (y4)
% 8 Highway MPG (y5)
% 9 Air Bags standard 0 = none, 1 = driver only, 2 = driver & passenger
% 10 Drive train type 0 = rear wheel drive 1 = front wheel drive 2 = all wheel drive
% 11 Number of cylinders (x1)
% 12 Engine size (liters) (x2)
% 13 Horsepower (maximum) (x3)
% 14 RPM (revs per minute at maximum horsepower) (x4)
% 15 Engine revolutions per mile (in highest gear) (x5)
% 16 Manual transmission available 0 = No, 1 = Yes
% 17 Fuel tank capacity (gallons) (x6)
% 18 Passenger capacity (persons) (x7)
% 19 Length (inches) (x8)
% 20 Wheelbase (inches) (x9)
% 21 Width (inches) (x10)
% 22 U-turn space (feet) (x11)
% 23 Rear seat room (inches) (x12)
% 24 Luggage capacity (cu. ft.) (x13)
% 25 Weight (pounds) (x14)
% 26 Domestic? 0 = non-U.S. manufacturer 1 = U.S. manufacturer
% we only consider the quantitative variables in the model.

% open the text file which contains the data in above url
fid = fopen('93cars.dat.txt');
% Read numbers and string as string into a cell array
rawdata=textscan(fid,'%s');
fclose(fid);
rawdata=rawdata{1};
% reshape into rectangular cell array (num row/colums hardcoded here)
numrows=93;
numcols=26;
rawdata=reshape(rawdata,numcols,numrows)';
num_obs_raw=size(rawdata,1);% number of observations, =93

% ignoring qualitative variables
quantitative=[4 5 6 7 8 11 12 13 14 15 17 18 19 20 21 22 23 24 25];
data=rawdata(:,quantitative);
num_var=size(data,2); % number of variables in the model: X's and y's

% the * in data shows missing value, we have to ignore them before using
% the data for our application. First we find rows with missing values
missing_idx=[];
for i=1:num_obs_raw
    for j=1:num_var
        if strcmp(data(i,j),'*')
            disp(['Observation ' num2str(i) ' is missing column ' num2str(j) '.']);
            missing_idx=[missing_idx i];
        end
    end
end
% now the rows with missing values are ommited 
data(missing_idx,:)=[];

% convert strings to numbers,
numericdata=str2double(data);

% selecting response variable (first 5 columns)
Y=numericdata(:,1:5);

%  selecting the explanatory variables (colums 6-19), since in our application we use normalized X,
%  and there are no qualitative variables in the model, the model will have no intercept, so the columns of
%  1's is not added to design matrix for this application.
XX=numericdata(:,6:19);

% save X and y in cars93.mat
save('93cars.mat','XX','Y')
disp('Saved data to 93cars.mat.')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Scatter plots and histograms of data %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Titles{1}={'y_1'};
Titles{2}={'y_2'};
Titles{3}={'y_3'};
Titles{4}={'y_4'};
Titles{5}={'y_5'};

tickm{1}=[0 20 40];
tickm{2}=[0 20 40 60];
tickm{3}=[0 20 40 60 80];
tickm{4}=[0 20 40];
tickm{5}=[0 20 40];
margin=2;

figure
for i=1:5 %rows
    miny=0;
    maxy=max(Y(:,i))+margin;
    for j=1:5 %columns
        minx=min(Y(:,j))-margin;
        maxx=max(Y(:,j))+margin;
        clear xlim ylim n xout
        px=0.134+((j-1)*0.16);
        py=0.828-((i-1)*0.10);
        subplot('Position',[px py 0.152 0.09])
        if i==j
            [n,xout]=hist(Y(:,i));
            hist(Y(:,i));
            h = findobj(gca,'Type','patch');
            set(h,'FaceColor','k','EdgeColor','k')
            axis([minx maxx miny maxy]);
            t=title(Titles{i});
            set(t,'Position',[(max(xout)+min(xout))/2 max(n)-10 17.32])
        else
            h2=plot(Y(:,j),Y(:,i),'ko');
            set(h2, 'MarkerSize', 2);
            axis([minx maxx miny maxy]);
        end;
        % tick marks and labels
        set(gca,'XTick',tickm{j});
        set(gca,'YTick',tickm{i});
        if i<5 set(gca,'XTickLabel',{});end;
        if j>1 set(gca,'YTickLabel',{});end;
    end
end
%pos1=[1 1 1680 400];
%set(gcf,'OuterPosition',pos1)
if savepic
    disp('Saving figure to eps file...');
    saveas(gcf,'figure4.eps');
end;