% application 2: 93cars data set
% This script performs group structured quantile regression the 93cars data set
% for many different random jitters to study the effect on the solution

clear all
numtrials=20;
savepic=true;
% set up list of tau
taulist= 0.5;%[0.1 0.5 0.9];%
epsilonlist=[0.000001 0.00001 0.0001];% size of jitter
% use seed to be able to duplicate this result
RandStream.setDefaultStream(RandStream('mt19937ar','seed',12345));


for ind2=1:numel(epsilonlist)
    epsilon=epsilonlist(ind2);
    disp(['Jitter size=' num2str(epsilon)]);

%%%%%%%%%%%%%%%%
% load data:
%%%%%%%%%%%%%%%%
load('93cars.mat')
% List of the loaded variables:
% Y: the matrix of 5 response variables
% XX: matrix of 14 explanatory variables

% now standardize X and y

Y_z=zscore(Y);
XX_z=zscore(XX);

% setting up y and X for application2, i.e., putting all y's along each
% other and making the block diangonal design matrix X.
y=[Y_z(:,1) ; Y_z(:,2) ;Y_z(:,3) ;Y_z(:,4) ;Y_z(:,5)];
X=blkdiag(XX_z,XX_z,XX_z,XX_z,XX_z);

numtau=numel(taulist);

% define groups:
numgroups=14;
numvar_y=5;
for i=1:numgroups
    groups{i}=i:numgroups:numgroups*numvar_y;
end

% defining the variables in the model
numvar=size(X,2);
numdata=size(X,1);


beta_selected=zeros(numtrials,numtau,numvar);
disp(['Start of ' num2str(numtrials) ' trials.']);
jitt=0;
datestr(now)

while jitt<numtrials
    jitt=jitt+1;
    disp(['Trial ' num2str(jitt) ' of ' num2str(numtrials)]);
    ind=0;
    
    % perform structured quantile regression for different tau
    while ind<numel(taulist)
        ind=ind+1;
        tau=taulist(ind);
        
        % add jitter to X
        for m=1:numvar
            jitter=randn(size(X(:,m)))*(std(X(:,m))*epsilon);
            X_jitt(:,m)=X(:,m)+jitter;
        end
        %add jitter to y
        y_jitt=y+(randn(size(y))*std(y)*epsilon);
        
        % set some input variables for the algorithm which are independent of \tau
        stopping.lambda=0;
        includetest=true;
        verbose=0;
        
        % NB: qrmix computes minimizers of ||A*x-b||_1 instead of ||y-X*beta||_1
        % (sign change = left/right change)
        weights.left=(2*tau)*ones(numdata,1);
        weights.right=(2*(1-tau))*ones(numdata,1);
        % compute solution path
        [xlist,lambdalist,u0list,u1list,w0list,w1list,succes]=qrmix(X_jitt,y_jitt,groups,stopping,verbose,weights,includetest);
        if ~succes
            % is qrmix is not succesful, then retry (with different jitter)
            disp('Trying again...')
            ind=ind-1;
            continue;
        else
            numpts=size(xlist,2);
            k=20;% number of interpolation points between each break point in xlist
            xlist_ext=interpolatelist(xlist,groups,k);
            [BIC tmp]=BICminimizer(xlist_ext,X_jitt,y_jitt,groups,tau,false);
            beta_selected(jitt,ind,:)=tmp;
        end;
    end
end

% compute relative differences with mean
beta_mean=mean(beta_selected);
differences=zeros(numtrials,numtau);
for i=1:numtrials
    for k=1:numtau
        tmp1=reshape(beta_selected(i,k,:),[],1);
        tmp2=reshape(beta_mean(1,k,:),[],1);
        tmp3=norm(tmp2);
        differences(i,k)=norm(tmp1-tmp2)/tmp3;
    end
end


    meandifferences(ind2,:)=mean(differences);
    stdevdifferences(ind2,:)=std(differences);
    
    
end%end loop over epsilon

timespent=toc;

% plot the mean difference
fontsize=24;
figure
linespeclist={'+k','*k','xk','ok'};
hold on;
for ind=1:numel(taulist)
plot(epsilonlist,meandifferences(:,ind),linespeclist{ind});
legendlist{ind}=['\tau=' num2str(taulist(ind))];
end
hold off;
ylim([10^(-7) 10^(-2)])
xlim([10^(-7) 10^(-3)])
set(gca,'XTick',taulist)
xlabel('Size of jitter','FontSize',fontsize)
ylabel('Mean difference','FontSize',fontsize)
set(gca,'yscale','log')
set(gca,'xscale','log')
set(gca,'XTick',epsilonlist)
set(gca,'YTick',[10^(-6) 10^(-4) 10^(-2)])
set(gca,'FontSize',fontsize);
legend(legendlist,'Location','SouthEast')
pbaspect([2 1 1])

if savepic saveas(gcf,'figure_jitter2.eps');end;

datestr(now)
disp('Done');