% application 2: CARS93 data set
% This script processes the 93cars data set and produces three figures
% 1: bic minimzation
% 2: solution path
% 3: two panels: the active groups (in the solution path) and the size
% final selected model (using bic)
% only last figured is saved to disk

clear all

savepic=true;% save picture at the end
epsilon=1/10000;%jitter constant
% use seed to be able to duplicate this result
RandStream.setDefaultStream(RandStream('mt19937ar','seed',12345));

%%%%%%%%%%%%%%%%
% load data:
%%%%%%%%%%%%%%%%
load('93cars.mat')
% List of the loaded variables:
% Y: the matrix of 5 response variables
% XX: matrix of 14 explanatory variables

% now standardize X and y

Y_z=zscore(Y);
XX_z=zscore(XX);

% setting up y and X for application2, i.e., putting all y's along each
% other and making the block diangonal design matrix X.
y=[Y_z(:,1) ; Y_z(:,2) ;Y_z(:,3) ;Y_z(:,4) ;Y_z(:,5)];
X=blkdiag(XX_z,XX_z,XX_z,XX_z,XX_z);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Group quantile regression %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add some small amount of random jitter to the data:
numcol=size(X,2);
for m=1:numcol
    jitter=randn(size(X(:,m)))*(std(X(:,m))*epsilon);
    X(:,m)=X(:,m)+jitter;
end
y=y+randn(size(y))*std(y)*epsilon;

% define groups:
numgroups=14;
numvar=5;
for i=1:numgroups
    groups{i}=i:numgroups:numgroups*numvar;
    disp(['Group ' num2str(i) ': ' num2str(groups{i})]);
end

% set quantile and stopping condition of algorithm
tau=0.5;
stopping.lambda=0;
% NB: qrmix computes minimizers of ||A*x-b||_1 instead of ||y-X*beta|\_1
% (sign change = left/right change)
weights.left=(2*tau)*ones(size(X,1),1);
weights.right=(2*(1-tau))*ones(size(X,1),1);
includetest=true;
verbose=0;
% compute solution path
disp('Start of qrmix...');
[xlist,lambdalist,u0list,u1list,w0list,w1list,succes]=qrmix(X,y,groups,stopping,verbose,weights,includetest);
disp('End of qrmix');

% compute number of break points returned by qrmix
numpts=size(xlist,2);

% calculate mixed norm R for break point in xlist
Rlist=zeros(1,numpts);
for i=1:numpts
    Rlist(i)=mixednorm(xlist(:,i),groups);
end

% Compute the  \ell_{\infty} norm of \hat{\beta}'s of each group:
infnorms=zeros(numgroups,numpts);
for i=1:numgroups
    for j=1:numpts
        infnorms(i,j)=norm(xlist(groups{i},j),inf);
    end
end

% determine whether the \ell_{\infty}-norm of each group is zero
% or not, along the solution path.
tolerance=0.00001;% a tolerance parameter
xlist_nzero=NaN(numgroups,numpts);
xlist_zero=NaN(numgroups,numpts);
for i=1:numgroups
    for j=1:numpts
        if abs(infnorms(i,j))>tolerance 
            xlist_nzero(i,j)=i;
            if j>0 xlist_nzero(i,j-1)=i; end;
            if j<numpts xlist_nzero(i,j+1)=i; end;
        else
            xlist_zero(i,j)=i;
        end
    end
end


% find the best model using BIC
k=20;% number of interpolation points between each break point in xlist
xlist_ext=interpolatelist(xlist,groups,k);
[BIC beta_selected]=BICminimizer(xlist_ext,X,y,groups,tau,true);

% find mixed norm R for selected beta:
R_selected=mixednorm(beta_selected,groups);
disp(['Mixed norm of selected beta= ' num2str(R_selected)]);


%find ell_{\infty} for each group in the estimated beta
for i=1:numgroups
    infnormgroups_beta_sel(i)=norm(beta_selected(groups{i}),inf);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot of the solution path
figure
plotstyles={'k',':k','--k','-.k','g',':g','--g','-.g','b',':b','--b','-.b','r',':r'};
for i=1:numgroups
    hold on;
    plot(Rlist,infnorms(i,:),plotstyles{i});
end
hold off
mx=max(Rlist(:));
axis([0 mx 0 0.7])
fontsize=10;
h= text('Interpreter','latex','HorizontalAlignment','center',...
            'FontName','Helvetica',...
            'String','$\|\hat\beta_{G_k}\|_\infty\,\,  (k=1,\ldots, 14)$',...
            'Rotation',90,...
            'Position',[-0.8 0.381],...
            'FontSize',fontsize);
set(h,'Interpreter','latex')
xlabel('R')
line([R_selected R_selected],[-max(abs(xlist(:)))/20 max(abs(xlist(:)))],'LineStyle',':','Color','k');
h= text('Interpreter','latex','HorizontalAlignment','center',...
            'FontName','Helvetica',...
            'String','$R_\mathrm{BIC}$',...
            'Position',[R_selected -max(abs(xlist(:)))/40],...
            'FontSize',fontsize);
set(h,'Interpreter','latex')
% legend
%position=[0.465 0.1333 0.046 0.501]; 
h1=legend('x_1','x_2','x_3','x_4','x_5','x_6','x_7',...
    'x_8','x_9','x_{10}','x_{11}','x_{12}','x_{13}','x_{14}');%,...
    %'Location',position);
title('Solution path as a function of R')



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% make figure with two panels:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
% panel left: presence/absence of each group along the solution path.
%subplot('Position',[0.6 0.423 0.3349 0.26])
subplot(1,2,1);
hold on
for i=1:numgroups
    plot(xlist_zero(i,:),Rlist,'-','LineWidth',4,'color',[0.75 0.75 0.75]);
    plot(xlist_nzero(i,:),Rlist,'-k','LineWidth',4);
end
hold off
axis([0 numgroups+1 0 mx])
set(gca,'XTick',1:14)
line([0 numgroups+1],[R_selected R_selected],'LineStyle',':','Color','k');
h= text('Interpreter','latex','HorizontalAlignment','center',...
            'FontName','Helvetica',...
            'String','$R_\mathrm{BIC}$',...
            'Position',[-1 R_selected],...
            'FontSize',fontsize);
set(h,'Interpreter','latex')
set(gca,'YTick',[0 1 3 5]);
xlabel('Explanatory variables index')
ylabel('R')
pbaspect([2 1 1])

% panel right, bottom: chosen \hat{\beta} values by BIC.
subplot(1,2,2);
mx2=max(infnormgroups_beta_sel);
%subplot('Position',[0.6 0.083 0.3349 0.26])
bar(infnormgroups_beta_sel,'k')
axis([0 numgroups+1 0 1.1*mx2])
set(gca,'XTick',1:numgroups)
xlabel('Explanatory variables index (k)')
fontsize=10;
h= text('Interpreter','latex','HorizontalAlignment','center',...
            'FontName','Helvetica',...
            'String','$\|\hat\beta_{G_k}\|_\infty$ for $R=R_\mathrm{BIC}$',...
            'Rotation',90,...
            'Position',[-2.32 0.194],...
            'FontSize',fontsize);
set(h,'Interpreter','latex')
pbaspect([2 1 1])
% save figure to file
if savepic
    disp('Saving figure to figure5.eps...');
    saveas(gcf,'figure5.eps','epsc');
end;
disp('Done.');



