% This script processes the 93cars data set and compare the 
% non-iterativre algorithm with the minimizers obtained by CVX

clear all

savepic=false;% save picture at the end
epsilon=1/10000;%jitter constant
% use seed to be able to duplicate this result
RandStream.setDefaultStream(RandStream('mt19937ar','seed',12345));

%%%%%%%%%%%%%%%%
% load data:
%%%%%%%%%%%%%%%%
load('93cars.mat')
% List of the loaded variables:
% Y: the matrix of 5 response variables
% XX: matrix of 14 explanatory variables

% now standardize X and y

Y_z=zscore(Y);
XX_z=zscore(XX);

% setting up y and X for application2, i.e., putting all y's along each
% other and making the block diangonal design matrix X.
y=[Y_z(:,1) ; Y_z(:,2) ;Y_z(:,3) ;Y_z(:,4) ;Y_z(:,5)];
X=blkdiag(XX_z,XX_z,XX_z,XX_z,XX_z);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Group quantile regression %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add some small amount of random jitter to the data:
numcol=size(X,2);
for m=1:numcol
    jitter=randn(size(X(:,m)))*(std(X(:,m))*epsilon);
    X(:,m)=X(:,m)+jitter;
end
y=y+randn(size(y))*std(y)*epsilon;

% define groups
% make string to use in cvx constraint:
numgroups=14;
numvar=5;
str='0';
for i=1:numgroups
    groups{i}=i:numgroups:numgroups*numvar;
    disp(['Group ' num2str(i) ': ' num2str(groups{i})]);
    
    str=[str '+norm(x(' num2str(i) ':14:' num2str(56+i) '),Inf)'];
end
disp(str);

% set quantile and stopping condition of algorithm
tau=0.5;
stopping.lambda=0;
% NB: qrmix computes minimizers of ||A*x-b||_1 instead of ||y-X*beta|\_1
% (sign change = left/right change)
weights.left=(2*tau)*ones(size(X,1),1);
weights.right=(2*(1-tau))*ones(size(X,1),1);
includetest=true;
verbose=0;
% compute solution path
disp('Start of qrmix...');
tic;
[xlist,lambdalist,u0list,u1list,w0list,w1list,succes]=qrmix(X,y,groups,stopping,verbose,weights,includetest);
timespent=toc;
disp('End of qrmix');
disp(['Time spent: ' num2str(timespent)]);

% compute number of break points returned by qrmix
numpts=size(xlist,2);

% calculate mixed norm R for break point in xlist
Rlist=zeros(1,numpts);
for i=1:numpts
    Rlist(i)=mixednorm(xlist(:,i),groups);
end

% Compute the  \ell_{\infty} norm of \hat{\beta}'s of each group:
infnorms=zeros(numgroups,numpts);
for i=1:numgroups
    for j=1:numpts
        infnorms(i,j)=norm(xlist(groups{i},j),inf);
    end
end

% compute loss (only correct for tau=0.5)
for i=1:numel(Rlist) loss1(i)=norm(X*xlist(:,i)-y,1);end;


% test cvx algorithm
disp('Starting CVX. This may take some time')
cvxminimizer=zeros(size(xlist));
cvxdist=zeros(1,numel(Rlist));
cvxloss=zeros(1,numel(Rlist));
tic;
for i=1:numel(Rlist)
    if mod(i,100)==1 disp(['Break point number ' num2str(i) ' of ' num2str(numel(Rlist))]);end;
R=Rlist(i);
xref=xlist(:,i);

cvx_begin quiet
    variable x(70)
    minimize( norm( X * x - y, 1 ) )
    subject to
        norm(x(1:14:57),Inf)+norm(x(2:14:58),Inf)+norm(x(3:14:59),Inf)+norm(x(4:14:60),Inf)+norm(x(5:14:61),Inf)+norm(x(6:14:62),Inf)+norm(x(7:14:63),Inf)+norm(x(8:14:64),Inf)+norm(x(9:14:65),Inf)+norm(x(10:14:66),Inf)+norm(x(11:14:67),Inf)+norm(x(12:14:68),Inf)+norm(x(13:14:69),Inf)+norm(x(14:14:70),Inf)<=R;
cvx_end

cvxminimizer(:,i)=x;
if norm(xref)>0
    cvxdist(i)=norm(x-xref)/norm(xref);
else
    cvxdist(i)=norm(x-xref);
end
end;
timespent=toc;
disp(['Time spent (CVX): ' num2str(timespent)]);
disp(['Time spent per CVX run: ' num2str(timespent/numel(Rlist)) 's. (' num2str(numel(Rlist)) ' runs).']);



%%%%%%%%%%%%%%%
% compute loss  (only correct for tau=0.5)
for i=1:numel(Rlist) loss2(i)=norm(X*cvxminimizer(:,i)-y,1);end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot iteration and trade-off curve
fontsize=10;
figure;

% cvx error
subplot(1,2,1)
plot(cvxdist,'-k');
axis([1 numel(Rlist) 0 1e-5]);
set(gca,'YTick',[0 0.2 0.4 0.6 0.8 1]*1e-5);
set(gca,'XTick',[200 400 600]);
% horizontal label
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica',...
        'String','$\mathsf{i}$',...
        'Position',[370 -0.15],...
        'FontSize',fontsize)
    % vertical label
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica',...
        'String','$\|\beta_{CVX}-\hat\beta\|_2/\|\hat\beta\|_2$',...
        'Position',[-14 0.5],...
        'Rotation',90,...
        'FontSize',fontsize)

pbaspect([2 1 1])

subplot(1,2,2)
plot(Rlist,loss1,'-k',Rlist,loss2,'--k');
axis([0 max(Rlist) 100 max(loss1)]);
set(gca,'YTick',[100 200 300]);
set(gca,'XTick',[0 1 2 3 4 5]);
%xlabel('R');
%ylabel('loss');
%title('Trade-off curve (loss as function of mixed norm)');
legend('Exact','CVX')
% horizontal label
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica',...
        'String','$\mathsf{R}$',...
        'Position',[1.02*(Rlist(1)+Rlist(end))/2 75],...
        'FontSize',fontsize)
    % vertical label
    text('Interpreter','latex','HorizontalAlignment','center',...
        'FontName','Helvetica',...
        'String','$\mathsf{Loss}$',...
        'Position',[-0.75 (loss1(1)+loss1(end))/2.2],...
        'Rotation',90,...
        'FontSize',fontsize)
    pbaspect([2 1 1])
if savepic saveas(gcf,'figure6c.eps');end;

figure;
plot(Rlist,loss2-loss1,'-k');
axis([0 max(Rlist) 0 max(loss2-loss1)]);
title('Error on loss')

disp(['Number of break points: ' num2str(numel(Rlist))]);
disp('Done.');