function [xlist,lambdalist,u0list,u1list,w0list,w1list,succes]=qrmix(A,b,groups,stopping,verbose,varargin)
% quantile regression with mixed-norm penalty

% stepwise (homotopy) algorithm for finding min_x
% ||Ax-b||_1+\lambda||x||_{1,\infty}
% Input: 
% A = a real matrix
% b = a real vector
% groups = a structure containing the lists of group members
% stopping =  a structure defining one or more stopping criteria, e.g.:
% stopping.numsteps: stop after this number of steps,
% stopping.lambda: stop after this lambda,
% stopping.mixnorm: stop when ||x|_{1,\infty} reaches this value
% stopping.misfit: stop when ||A*x-b||_1 (or generalization) reaches this
% value (or less)
% stopping.time: stop after this number of seconds, 
% stopping.numzeroeq: stop after this number of equations in A*x=b are satisfied
% stopping.numactivegroups: stop if this number of active groups is reached
% If these fields do not exists, then defaults are created.
% verbose = an integer 
%  -1= print nothing at all (even when algorithm stops on error)
%   0= print no information except error
%   1= print ||x||_{1,\infty} and lambda interval of each breakpoint, and reason
%   why algorithm stopped. At start print stopping criteria
%   2= above plus detailed information in each step

% qrmix(A,b,groups,stopping,verbose,weigths): use weights.left and weights.right
% (lists of strictly positive numbers) in the slope of definition of
% ||A*x-b||_1-> \sum_i [-weights.left(i)(A*x-b)_i if (A*x-b)_i<=0
%                       weights.right(i)(A*x-b)_i if (A*x-b)_i>=0]
% qrmix(A,b,groups,stopping,verbose,weigths,includetest): if includetest=true then
% verify the solution at each breakpoint by testing that it satisfies
% variational equations (default is true). Setting it to false speeds up
% the algorithm a bit.

% Output:
% xlist = a list of minimizers (break points), starting at x=[0;0;...;0]
% lambdalist = a list of lambdas (between breakpoints) 
% u0list, u1list = list of u0 and u1 where u0+u1/lambda is in the 
%                                      subdifferential of ||x||_{1,\infty}
% w0list, w1list = list of w0 and w1 where w0+lambda*w1 is in the 
%                                      subdifferential of ||A*x-b||_1
% succes = true if succesful, false if algorithm fails

% A numerical tolerance is used to check which variables are zero (equal).
% (See below).

% Last modified by Ignace Loris (igloris@ulb.ac.be) on 7/01/2013

ifthendisp(verbose>1,'==============');
ifthendisp(verbose>1,'Starting qrmix');
ifthendisp(verbose>1,'==============');

% set up some variables
tic;
tolerance=10^(-9);%used for u and w (approx =1)
toleranceb=tolerance*norm(b);%used for "b"
tolerancex=toleranceb/norm(A);%used for "x"
tolerancelambda=tolerance*norm(A);%used for lambda
succes=true;
xlist=[];
lambdalist=[];
u0list=[];
u1list=[];
w0list=[];
w1list=[];

numvar=size(A,2);
numdata=size(A,1);
numgroups=numel(groups);
for i=1:numgroups numvargroup{i}=numel(groups{i});end;
% check consistency
if numdata~=numel(b)
    ifthendisp(verbose>-1,'Size of b does not match size of A. Stopping!');
    succes=false;
    return;
end;
% check for exceptional case
if numel(find(abs(b)<=toleranceb))>0
    ifthendisp(verbose>-1,'Cannot handle case when b has zero entries. Stopping.');
    succes=false;
    return;
end;

% handle optional arguments
optargin = size(varargin,2);
if optargin>0
    weights=varargin{1};
    if ~isfield(weights,'left') || ~isfield(weights,'right') || ...
            size(weights.left,2)~=1 || size(weights.right,2)~=1 || ...
            size(weights.left,1)~=numdata || size(weights.right,1)~=numdata
        ifthendisp(verbose>-1,'Size of weights does not match size of A. Stopping!');
    return;
    end
else
    % default values
    weights.left=ones(numdata,1);
    weights.right=ones(numdata,1);
end
if optargin<2
    includetest=true;%default
else
    includetest=varargin{2};
end


% set default stopping criteria (if necessary)
continuealgorithm=true;
if ~isfield(stopping,'numsteps') stopping.numsteps=Inf;end;%will never be reached
if ~isfield(stopping,'lambda') stopping.lambda=tolerancelambda;end;
if ~isfield(stopping,'mixnorm') stopping.mixnorm=Inf;end;%will never be reached
if ~isfield(stopping,'misfit') stopping.misfit=toleranceb;end;
if ~isfield(stopping,'time') stopping.time=Inf;end;% will never be reached
if ~isfield(stopping,'numzeroeq') stopping.numzeroeq=numdata;end;
if ~isfield(stopping,'numactivegroups') stopping.numactivegroups=numgroups+1;end;% will never be reached
ifthendisp(verbose>0,['Stopping criteria are: ' structure2tex(stopping,'|')]);




% first step (outside loop)
stepctr=0;
ifthendisp(verbose>1,'---------------');
ifthendisp(verbose>1,['Step number ' num2str(stepctr) ':']);

x=zeros(numvar,1);% minimizer
r=A*x-b;% misfit vector
% subdifferential of ||r||_1
w0=weightedsign(r,weights.left,weights.right);
w1=zeros(numdata,1);
% subdifferential of ||x|_{1,\infty}
u0=zeros(numvar,1);
u1=-A'*w0;
lambda=mixednormdual(u1,groups);% smallest lambda for x=0
R=mixednorm(x,groups);% ||x|_{1,\infty}
misfit=misfitfunc(r,weights.left,weights.right);% ||A*x-b||_1 or \sum_i [weights.left(i)(A*x-b)_i if (A*x-b)_i<=0
%                       weights.right(i)(A*x-b)_i if (A*x-b)_i>=0]
zeroeq=find(abs(r)<=toleranceb);% equations in A*x-b=0 that are satisfied
nonzeroeq=find(abs(r)>toleranceb);% equations in A*x-b=0 that are not satisfied
numzeroeq=numel(zeroeq);% number of eq. in A*x-b=0 that are satisfied
activegroups=[];
numactivegroups=numel(activegroups);
% following should become structures
% maxvars=find(abs(abs(x)-R)<=tolerance*R);% components of x equal to ||x|_\infty (in absolute value)
% nonmaxvars=find(abs(abs(x)-R)>tolerance*R);% components of x smaller than ||x|_\infty (in absolute value)
% nummaxvars=numel(maxvars);% number of components of x equal to ||x|_\infty (in absolute value)

ifthendisp(verbose>1,[num2str(stepctr) ': Break point x is: ' num2str(x')]);
ifthendisp(verbose>1,[num2str(stepctr) ': Misfit r at break point x is: ' num2str(r')]);
ifthendisp(verbose>1,[num2str(stepctr) ': w=[' num2str(w0') ']+[' num2str(w1') ']*lambda']);
ifthendisp(verbose>1,[num2str(stepctr) ': u=[' num2str(u0') ']+[' num2str(u1') ']*lambda^(-1)']);
ifthendisp(verbose>1,[num2str(stepctr) ': lambda found from ||u||_1=1.']);
% check if more than 1 new active group (\|u_G\|_1=1)
u=u0+u1/lambda;
nonactivegroups=1:numgroups;
newgroups=[];
for i=1:numel(nonactivegroups)
        ind=groups{nonactivegroups(i)};
        if norm(u(ind),1)==1 newgroups=[newgroups nonactivegroups(i)];end;
end
if numel(newgroups)>1
    ifthendisp(verbose>1,[num2str(stepctr) ': More than 1 new active group. Groups: ' num2str(newgroups) '.']);
    ifthendisp(verbose>1,[num2str(stepctr) ': Cannot handle this case. Stopping...']);
    continuealgorithm=false;
    succes=false;
end
% check that members of new active group start out maximal
numnonactivegroups=numel(nonactivegroups);
for i=1:numnonactivegroups
dummy=find(abs(u(groups{nonactivegroups(i)}))<=tolerance);
if numel(dummy)>0
    ifthendisp(verbose>1,[num2str(stepctr) ...
        ': u of group ' num2str(nonactivegroups(i)) ' has zero component(s) at entry in active set.' ...
        ' Zero component(s) is/are: ' num2str(groups{nonactivegroups(i)}(dummy)) '.' ...
        ' Cannot handle this case. Stopping.']);
    continuealgorithm=false;
    succes=false;
end;
end;
ifthendisp(verbose>1,[num2str(stepctr) ': New lambda=' num2str(lambda)]);
ifthendisp(verbose>0,[num2str(stepctr) ': lambda interval at this break point (||x||_mix=' num2str(mixednorm(x,groups)) '): ' ...
    'Inf>=lambda>=' num2str(lambda)])

% verify variational equations
if includetest
    tmp1=verifyminimizergroup(A,b,groups,x,lambda,u0+u1/lambda,w0+w1*lambda,weights.left,weights.right);
    if tmp1(1)>tolerancelambda || tmp1(2)>tolerance ||  tmp1(3)>tolerance
        ifthendisp(verbose>-1,[num2str(stepctr) ': Someting is wrong! ' ...
            num2str(tmp1) '>' num2str([tolerancelambda tolerance tolerance])]);
        ifthendisp(verbose>-1,[num2str(stepctr) ': Stopping...']);
        continuealgorithm=false;
        succes=false;
    end
end

% append to output lists:
xlist=[xlist,x];
lambdalist=[lambdalist,lambda];
u0list=[u0list,u0];
u1list=[u1list,u1];
w0list=[w0list,w0];
w1list=[w1list,w1];

% check stopping conditions
timespent=toc;
if timespent>=stopping.time
    continuealgorithm=false;
    ifthendisp(verbose>0,['Maximum time (' num2str(timespent) '>=' num2str(stopping.time) ' seconds) exceeded.']);
end
if stepctr>=stopping.numsteps
    continuealgorithm=false;
    ifthendisp(verbose>0,['Maximum number of steps (' num2str(stepctr) '>=' num2str(stopping.numsteps) ') exceeded.']);
end;
if lambda<=stopping.lambda
    continuealgorithm=false;
    ifthendisp(verbose>0,['Minimal value of lambda  (' num2str(lambda) '<=' num2str(stopping.lambda) ') exceeded.']);
end;
if R>=stopping.mixnorm
    continuealgorithm=false;
    ifthendisp(verbose>0,['Maximum value of ||x||_{1,\infty}  (' num2str(R) '>=' num2str(stopping.mixnorm) ') exceeded.']);
end;
if misfit<=stopping.misfit
    continuealgorithm=false;
    ifthendisp(verbose>0,['Minimum value of ||A*x-b||_1  (' num2str(misfit) '<=' num2str(stopping.misfit) ') exceeded.']);
end;
if numzeroeq>=stopping.numzeroeq
    continuealgorithm=false;
    ifthendisp(verbose>0,['Maximum number of equations satisfied (' num2str(numzeroeq) '>=' num2str(stopping.numzeroeq) ') exceeded.']);
end;
if numactivegroups>=stopping.numactivegroups
    continuealgorithm=false;
    ifthendisp(verbose>0,['Number of active groups (' num2str(numactivegroups) '<=' num2str(stopping.numactivegroups) ') exceeded.']);
end;



% enter main loop. Each step corresponds to a break point
while continuealgorithm
    stepctr=stepctr+1;
    ifthendisp(verbose>1,'---------------');
    ifthendisp(verbose>1,['Step number ' num2str(stepctr) ':']);
    
    
    u=u0+u1/lambda;
    % determine active groups
    % and maximal components in each group
    tmp1=group1norms(u,groups);
    tmp2=max(tmp1);
    activegroups=find(abs(tmp1-tmp2)<=tolerance);
    nonactivegroups=setdiff(1:numgroups,activegroups);
    numnonactivegroups=numel(nonactivegroups);
    numactivegroups=numel(activegroups);
    ifthendisp(verbose>1,...
        [num2str(stepctr) ': Active groups are: {' num2str(activegroups) ...
        '} (' num2str(numactivegroups) ' out of ' num2str(numgroups) ')']);
    
    for i=1:numgroups
        tmp1=find(abs(u(groups{i}))<=tolerance);
        nonmaxvars{i}=groups{i}(tmp1);% these will be the comp. of x that are not maximal
        maxvars{i}=setdiff(groups{i},nonmaxvars{i});
        nummaxvars{i}=numel(maxvars{i});
        ifthendisp(verbose>1,...
            [num2str(stepctr) ':- Maximal components of group ' num2str(i) ' are: {' num2str(maxvars{i}) ...
            '} (' num2str(nummaxvars{i}) ' of ' num2str(numvargroup{i}) ')']);
    end
    
    %compute x of the form x=x0+x1*R, where R=\|x\|_{1,\infty}
    x0=zeros(numvar,1);
    x1=zeros(numvar,1);
    
    % set up linear system for calculating x0 and x1
    mat=zeros(numvar,numvar);
    rhs=zeros(numvar,2);
    ctr=0;
    for i=1:numactivegroups
        mx=maxvars{activegroups(i)};
        for j=2:numel(mx)
            ctr=ctr+1;
            mat(ctr,mx(1))=sign(u(mx(1)));
            mat(ctr,mx(j))=-sign(u(mx(j)));
            rhs(ctr,:)=[0,0];
        end
    end
    for i=1:numnonactivegroups
        grp=groups{nonactivegroups(i)};
        for j=1:numel(grp)
        ctr=ctr+1;
        mat(ctr,grp(j))=1;
        rhs(ctr,:)=[0,0];
        end;
    end;
    % include eq. saying ||x||_{1,\infty}=R
    ctr=ctr+1;
    for i=1:numactivegroups
        mx=maxvars{activegroups(i)};
        mat(ctr,mx(1))=sign(u(mx(1)));
    end
    rhs(ctr,:)=[0,1];
    % include equations already satisfied
    %disp([num2str(size(mat)) '---' num2str(ctr+1) '---' num2str(zeroeq)]);
    mat(ctr+1:end,:)=A(zeroeq,:);
    rhs(ctr+1:end,:)=[b(zeroeq) , zeros(numel(zeroeq),1)];
    
    tmp1=mat\rhs;
    x0=tmp1(:,1);
    x1=tmp1(:,2);
    ifthendisp(verbose>1,[num2str(stepctr) ': x=[' num2str(x0') ']+R*[' num2str(x1') ']']);
    
    % compute r of the form r=r0+r1*R:
    r0=A*x0-b;
    r1=A*x1;
    % find at which value of R a component of r becomes zero first:
    tmp1=-r0./r1;
    % disregrard the equations that were already zero
    tmp1(zeroeq)=Inf;
    %disregard the ones smaller than previous R:
    tmp1(tmp1<=R+tolerance*R)=Inf;
    
    tmp4=tmp1;% main list
    
    % for each active group: find where a maximal component changes sign
    % (becomes zero)
    clear tmp5;
    for i=1:numactivegroups
        ind=activegroups(i);
        tmp5{i}=-x0(maxvars{ind})./x1(maxvars{ind});
        %disregard the ones smaller than previous R:
        tmp5{i}(tmp5{i}<=R+tolerance*R)=Inf;
        tmp5{i}(tmp5{i}<=R+tolerance*R)=Inf;
        tmp4=[tmp4;tmp5{i}];
    end 
    
    
    % for each active group:
    % find at which value of R, does a nonmax component of x equal the max
    % components:
    clear tmp3plus tmp3min;
    for i=1:numactivegroups
        ind=activegroups(i);
        %maxvars{ind}
        %nonmaxvars{ind}
        tmp3plus{i}=(x0(maxvars{ind}(1))-x0(nonmaxvars{ind}))./(x1(nonmaxvars{ind})-x1(maxvars{ind}(1)));
        tmp3min{i}=(-x0(maxvars{ind}(1))-x0(nonmaxvars{ind}))./(x1(nonmaxvars{ind})+x1(maxvars{ind}(1)));
        %disregard the ones smaller than previous R:
        tmp3plus{i}(tmp3plus{i}<=R+tolerance*R)=Inf;
        tmp3min{i}(tmp3min{i}<=R+tolerance*R)=Inf;
        tmp4=[tmp4;tmp3plus{i};tmp3min{i}];
    end 
    
    
    % find smallest positive R:
    R=min(tmp4);
    inr=find(tmp1==R);
    clear inx;
    for i=1:numactivegroups
        inx{i}=find(tmp5{i}==R);
        if numel(inx{i})>0% this group has become inactive
            ifthendisp(verbose>1,[num2str(stepctr) ': R found from nonzero -> zero component of x']);
        end
    end 
    
    clear inmaxplus inmaxmin;
    for i=1:numactivegroups
        inmaxplus{i}=find(tmp3plus{i}==R);
        inmaxmin{i}=find(tmp3min{i}==R);
    end
    if numel(inr)>0
        ifthendisp(verbose>1,[num2str(stepctr) ': R found from new zero component of r.']);
    end;
    for i=1:numactivegroups
        ind=activegroups(i);
        if numel(inmaxplus{i})>0
            ifthendisp(verbose>1,[num2str(stepctr) ...
                ': R found from non maximal component becoming maximal (+) in group ' num2str(ind) '.']);
            %update maximal set:
            nonmaxvars{ind}=setdiff(nonmaxvars{ind},nonmaxvars{ind}(inmaxplus{i}));
            maxvars{ind}=setdiff(groups{ind},nonmaxvars{ind});
            nummaxvars{ind}=numel(maxvars{ind});
            ifthendisp(verbose>1,...
                 [num2str(stepctr) ':- Maximal components of (active) group ' ...
                 num2str(ind) ' are: {' num2str(maxvars{ind}) ...
            '} (' num2str(nummaxvars{ind}) ' of ' num2str(numvargroup{ind}) ')']);
        end;
        if numel(inmaxmin{i})>0
            ifthendisp(verbose>1,[num2str(stepctr) ...
                ': R found from non maximal component becoming maximal (-) in group ' num2str(ind) '.']);
            %update maximal set:
            nonmaxvars{ind}=setdiff(nonmaxvars{ind},nonmaxvars{ind}(inmaxmin{i}));
            maxvars{ind}=setdiff(groups{ind},nonmaxvars{ind});
            nummaxvars{ind}=numel(maxvars{ind});
            ifthendisp(verbose>1,...
                 [num2str(stepctr) ':- Maximal components of (active) group ' ...
                 num2str(ind) ' are: {' num2str(maxvars{ind}) ...
            '} (' num2str(nummaxvars{ind}) ' of ' num2str(numvargroup{ind}) ')']);
        end;
    end;
    ifthendisp(verbose>1,[num2str(stepctr) ': R=' num2str(R)]);
    % update x:
    x=x0+x1*R;
    ifthendisp(verbose>1,[num2str(stepctr) ': Break point x is: ' num2str(x')]);    
    % update list of zero eqns
    if numel(find(tmp1==R))>1
        ifthendisp(verbose>-1,[num2str(stepctr) ': More than 1 new zero equation in this step (at components: ' ...
            num2str(find(tmp1==R)') '). Stopping!']);
        ifthendisp(verbose>-1,[num2str(stepctr) ': r=' num2str((A*x-b)') '.']);
        continuealgorithm=false;
        succes=false;
        return;
    end;
    % equations that are satisfied
    newzeroeq=find(tmp1==R);
    zeroeq=[zeroeq,newzeroeq];% or after updating r directly by find(abs(r)<=toleranceb);
    if numel(newzeroeq)>0
        ifthendisp(verbose>1,[num2str(stepctr) ': New equation that is satisfied: ' num2str(newzeroeq)]);
    end
    % update r
    r=A*x-b;
    ifthendisp(verbose>1,[num2str(stepctr) ': Misfit r at break point x is: ' num2str(r')]);
    nonzeroeq=find(abs(r)>toleranceb);% equations in A*x-b=0 that are not satisfied 
    if numel(nonzeroeq)<numdata-numel(zeroeq)
        ifthendisp(verbose>-1,[num2str(stepctr) ': There are more equations satisfied than ' ...
            'expected: ' num2str(setdiff(1:numdata,nonzeroeq)) ' are (numerically) satisfied, but only eqn(s) ' ...
            num2str(zeroeq) ' is/are accounted for.']);
        ifthendisp(verbose>-1,[num2str(stepctr) ...
            ': Probably R is determined by two conditions simultaneously, which this code cannot handle']);
        ifthendisp(verbose>-1,[num2str(stepctr) ': Stopping...']);
        continuealgorithm=false;
        succes=false;
        return;
        % zeroeq + nonzeroeq does not give all eqns...
    end;
    
    numzeroeq=numel(zeroeq);
    ifthendisp(verbose>1,[num2str(stepctr) ': Equations that are satisfied: {' num2str(zeroeq) ...
        '} (' num2str(numzeroeq) ' out of ' num2str(numdata) ')']);
    misfit=misfitfunc(r,weights.left,weights.right);%||A*x-b||_1 or generalization
    
    % update active groups because active may become nonactive!
    oldactivegroups=activegroups;
    activegroups=[];
    for i=1:numgroups
        if max(abs(x(groups{i})))>tolerancex%active group
            activegroups=[activegroups i];
        else% nonactive group: make sure all components are set as maximal
            maxvars{i}=groups{i};
        end;
    end
    numactivegroups=numel(activegroups);
    nonactivegroups=setdiff(1:numgroups,activegroups);
    if numactivegroups<numel(oldactivegroups)
        ifthendisp(verbose>1,[num2str(stepctr) ': Active group(s) ' ...
            num2str(setdiff(oldactivegroups,activegroups)) ...
            ' became nonactive.']);
    end;   
    ifthendisp(verbose>1,...
        [num2str(stepctr) ': Active groups are: {' num2str(activegroups) ...
        '} (' num2str(numactivegroups) ' out of ' num2str(numgroups) ')']);

    
    % update u and w
    % initialize
    u0=zeros(numvar,1);
    u1=zeros(numvar,1);
    w0=zeros(numdata,1);
    w1=zeros(numdata,1);
    % set easy parts:
    for i=1:numactivegroups
        ind=activegroups(i);
        u0(nonmaxvars{ind})=0;%non max vars of active groups
        u1(nonmaxvars{ind})=0;%non max vars of active groups
    end
    w0(nonzeroeq)=weightedsign(r(nonzeroeq),weights.left(nonzeroeq),weights.right(nonzeroeq));
    w1(nonzeroeq)=0;
    % calculate difficult parts by solving linear system A'*w+lambda*u=0
    % and lin comb of u(maxvars) =1
    Id=eye(numvar);
    totmaxvars=0;%# of maximal variables in *all* groups (also non active)
    allmaxvars=[];
    for i=1:numgroups 
        allmaxvars=[allmaxvars, maxvars{i}];
        totmaxvars=totmaxvars+numel(maxvars{i});
    end;
    mat=[A(zeroeq,:)' Id(:,allmaxvars)];
    rhs=-A(nonzeroeq,:)'*[w0(nonzeroeq), w1(nonzeroeq)];
    for i=1:numactivegroups % add a line for each active group
        ind=activegroups(i);
        tmp=zeros(1,numvar);
        tmp(maxvars{ind})=sign(x(maxvars{ind}))';
        tmp=tmp(allmaxvars);
        mat=[mat;zeros(1,numzeroeq) tmp];
        rhs=[rhs;0 1];
    end;
    
    % matrix should be square otherwise something is wrong
    ifthendisp(verbose>2,[num2str(stepctr) ': Size of intermediate matrix is: ' num2str(size(mat)) ]);
    if size(mat,1)~=size(mat,2)
        ifthendisp(verbose>-1,[num2str(stepctr) ': Intermediate matrix is not square. Something is wrong! Stopping...']);
        continuealgorithm=false;
        succes=false;
        return;
    end
    
    sol=mat\rhs;
    w0(zeroeq)=sol(1:numzeroeq,1);
    w1(zeroeq)=sol(1:numzeroeq,2);
    u0(allmaxvars)=sol(numzeroeq+1:end,2);
    u1(allmaxvars)=sol(numzeroeq+1:end,1);
    ifthendisp(verbose>1,[num2str(stepctr) ': w=[' num2str(w0') ']+[' num2str(w1') ']*lambda']);
    ifthendisp(verbose>1,[num2str(stepctr) ': u=[' num2str(u0') ']+[' num2str(u1') ']*lambda^(-1)']);
    % double check A'*w+lambda*u=0 and extra eqns for u's!!<----
    % verify variational equations for previous value of lambda
    %-------------------> append u w to list and compare with noisy version
    if includetest
        tmp1=verifyminimizergroup(A,b,groups,x,lambda,u0+u1/lambda,w0+w1*lambda,weights.left,weights.right);
        if tmp1(1)>tolerancelambda || tmp1(2)>tolerance ||  tmp1(3)>tolerance
            ifthendisp(verbose>-1,[num2str(stepctr) ': Someting is wrong! ' ...
                num2str(tmp1) '>' num2str([tolerancelambda tolerance tolerance])]);
            ifthendisp(verbose>-1,[num2str(stepctr) ': Stopping...']);
            continuealgorithm=false;
            succes=false;
            return;
        end
    end
    
    
    % calculate value of lambda by requiring that w and u are
    % subdifferentials of ||r||_1 and ||x||_mix respectively
    % lambda such that w=weights.right
    tmpw1=(weights.right(zeroeq)-w0(zeroeq))./w1(zeroeq);
    %disregard the negative ones:
    tmpw1(tmpw1<=tolerancelambda)=-Inf;
    %disregard the ones larger than current lambda:
    tmpw1(tmpw1>=lambda-tolerance*lambda)=-Inf;%
    % lambda such that w=-weights.left
    tmpw2=(-weights.left(zeroeq)-w0(zeroeq))./w1(zeroeq);
    %disregard the negative ones:
    tmpw2(tmpw2<tolerancelambda)=-Inf;
    %disregard the ones larger than current lambda:
    tmpw2(tmpw2>=lambda-tolerance*lambda)=-Inf;%
    
    
    tmp=[tmpw1;tmpw2];% main list
    % lambda where u=0 for active group
    tmpu1all=[];
    clear tmpu1;
    for i=1:numactivegroups
        tmpu1{i}=-u1(maxvars{activegroups(i)})./u0(maxvars{activegroups(i)});
        %disregard the negative ones:
        tmpu1{i}(tmpu1{i}<tolerancelambda)=-Inf;
        %disregard the ones larger than current lambda:
        tmpu1{i}(tmpu1{i}>=lambda-tolerance*lambda)=-Inf;
        % append to main list
        tmpu1all=[tmpu1all;tmpu1{i}];
    end
    tmp=[tmp;tmpu1all];
    
    
    % lambda where ||u||_1=1 for non-active group (becomes active)
    tmpu2all=[];
    clear tmpu2;
    for i=1:numel(nonactivegroups)
        ind=groups{nonactivegroups(i)};
        a1=u0(ind);
        a2=u1(ind);
        [mu1,mu2]=findl1ballintersection(a1,a2,1);
        tmpu2{i}=[1/mu1;1/mu2];
        %disregard the negative ones:
        tmpu2{i}(tmpu2{i}<tolerancelambda)=-Inf;
        %disregard the ones larger than current lambda:
        tmpu2{i}(tmpu2{i}>=lambda-tolerance*lambda)=-Inf;
        % append to main list
        tmpu2all=[tmpu2all;tmpu2{i}];
    end
    tmp=[tmp;tmpu2all];
    
    % determine next lambda
    lambdaold=lambda;% used in message only
    lambda=max(tmp);
    
    if lambda==-Inf
        lambda=0;
        ifthendisp(verbose>1,[num2str(stepctr) ': lambda set to 0.']);
    end;
    
    inu1=find(tmpu1all==lambda);
    inu2=find(tmpu2all==lambda);
    inw1=find(tmpw1==lambda);
    inw2=find(tmpw2==lambda);
    newgroup=false;
    newnonmax=false;
    if numel(inu1)>0
        %ifthendisp(verbose>1,[num2str(stepctr) ': Lambda found from u=0 (at position ).']);
        for i=1:numactivegroups
            j=find(tmpu1{i}==lambda);
            if numel(j)>0
                ifthendisp(verbose>1,[num2str(stepctr) ...
                    ': Lambda found from u=0: in group ' num2str(activegroups(i)) ...
                    ' at position ' num2str(maxvars{activegroups(i)}(j)) '.']);
                newnonmax=true;
            end
            if numel(j)>1 
                ifthendisp(verbose>1,[num2str(stepctr) ...
                    'Several u become zero at the same time... Cannot handle this case. Stopping ']);
                continuealgorithm=false;
                succes=false;
            end;
        end
    end;
    if numel(inu2)>0
        numnewgroups=0;
        for i=1:numel(nonactivegroups)
            j=find(tmpu2{i}==lambda);
            if numel(j)>0
                ifthendisp(verbose>1,[num2str(stepctr) ': Lambda found from ||u_G||_1=1. New active group: ' ...
                    num2str(nonactivegroups(i)) '.']);
                newgroup=true;
                numnewgroups=numnewgroups+1;
                % check that u has no zero components to start with.
                u=u0+u1/lambda;
                dummy=find(abs(u(groups{nonactivegroups(i)}))<=tolerance);
                if numel(dummy)>0
                    ifthendisp(verbose>1,[num2str(stepctr) ...
                        ': u of group ' num2str(nonactivegroups(i)) ' has zero component(s) at entry in active set.' ...
                        ' Zero component(s) is/are: ' num2str(groups{nonactivegroups(i)}(dummy)) '.' ...
                        ' Cannot handle this case. Stopping.']);
                    continuealgorithm=false;
                    succes=false;
                end
            end
        end
        if numnewgroups>1 
            ifthendisp(verbose>1,[num2str(stepctr) ...
                ': More than 1 new active group. Cannot handle this case. Stopping.']);
            continuealgorithm=false;
            succes=false;
        end;
    end;
    if numel(inw1)>0 || numel(inw2)>0
        ifthendisp(verbose>1,[num2str(stepctr) ': Lambda found from w=+/-1.']);
        if numel([inw1,inw2])>1
            ifthendisp(verbose>-1,[num2str(stepctr) ': More than one w becomes +/-1 at the same time... Stopping']);
            succes=false;
            continuealgorithm=false;
        else
            ifthendisp(verbose>1,[num2str(stepctr) ': Removing ' num2str(zeroeq([inw1;inw2])') ' from list of zero equations.']);
            zeroeq=setdiff(zeroeq,zeroeq([inw1;inw2]));
        end
    end;
    ifthendisp(verbose>1,[num2str(stepctr) ': New lambda=' num2str(lambda)]);
    ifthendisp(verbose>0,[num2str(stepctr) ': lambda interval at this break point (||x||_mix=' ...
        num2str(mixednorm(x,groups)) '): ' ...
        num2str(lambdaold) '>=lambda>=' num2str(lambda)])
    
    
    
    % append to output lists:
    xlist=[xlist,x];
    lambdalist=[lambdalist,lambda];
    u0list=[u0list,u0];
    u1list=[u1list,u1];
    w0list=[w0list,w0];
    w1list=[w1list,w1];
    

    % verify variational equations
    if includetest
        %disp('end:');
        tmp1=verifyminimizergroup(A,b,groups,x,lambda,u0+u1/lambda,w0+w1*lambda,weights.left,weights.right);
        if tmp1(1)>tolerancelambda || tmp1(2)>tolerance ||  tmp1(3)>tolerance
            ifthendisp(verbose>-1,[num2str(stepctr) ': Someting is wrong! ' ...
                num2str(tmp1) '>' num2str([tolerancelambda tolerance tolerance])]);
            ifthendisp(verbose>-1,[num2str(stepctr) ': Stopping...']);
            continuealgorithm=false;
            succes=false;
        end
    end
    
    if newgroup && newnonmax
        continuealgorithm=false;
        succes=false;
        ifthendisp(verbose>-1,[num2str(stepctr) ': New group AND u=0 at the same time. Cannot handle this case. Stopping...']);
    end;
    
    
    % check stopping conditions
    timespent=toc;
    if timespent>=stopping.time
        continuealgorithm=false;
        ifthendisp(verbose>0,['Maximum time (' num2str(timespent) '>=' num2str(stopping.time) ' seconds) exceeded.']);
    end
    if stepctr>=stopping.numsteps
        continuealgorithm=false;
        ifthendisp(verbose>0,['Maximum number of steps (' num2str(stepctr) '>=' num2str(stopping.numsteps) ') exceeded.']);
    end;
    if lambda<=stopping.lambda
        continuealgorithm=false;
        ifthendisp(verbose>0,['Minimal value of lambda  (' num2str(lambda) '<=' num2str(stopping.lambda) ') exceeded.']);
    end;
    if R>=stopping.mixnorm
        continuealgorithm=false;
        ifthendisp(verbose>0,['Maximum value of ||x||_\infty  (' num2str(R) '>=' num2str(stopping.mixnorm) ') exceeded.']);
    end;
    if misfit<=stopping.misfit
        continuealgorithm=false;
        ifthendisp(verbose>0,['Minimum value of ||A*x-b||_1  (' num2str(misfit) '<=' num2str(stopping.misfit) ') exceeded.']);
    end;
    if numzeroeq>=stopping.numzeroeq
        continuealgorithm=false;
        ifthendisp(verbose>0,['Maximum number of equations satisfied (' num2str(numzeroeq) '>=' num2str(stopping.numzeroeq) ') exceeded.']);
    end;
    if numactivegroups>=stopping.numactivegroups
        continuealgorithm=false;
        ifthendisp(verbose>0,['Number of active groups (' num2str(numactivegroups) '<=' num2str(stopping.numactivegroups) ') exceeded.']);
    end;
    
end;

if succes
    ifthendisp(verbose>1,'=======================');
    ifthendisp(verbose>1,'qrmix ended succesfully');
    ifthendisp(verbose>1,'=======================');
else
    ifthendisp(verbose>1,'qrmix failed to find minimizer.');
end;









%%%%%%%%%%%%%%%%%%%%%%%%%%
% Some helper functions: %
%%%%%%%%%%%%%%%%%%%%%%%%%%
function ifthendisp(test,message)
if test
    disp(message);
end

function f=weightedsign(u,wleft,wright)
f=zeros(size(u));
for i=1:numel(u)
    if u(i)<0
        f(i)=-wleft(i);
    elseif u(i)>0
        f(i)=wright(i);
    end
end

function f=misfitfunc(r,left,right)
f=dot(weightedsign(r,left,right),r);